package controllers

import (
	"encoding/json"
	"eta/eta_index_lib/logic"
	"eta/eta_index_lib/models"
	"eta/eta_index_lib/services"
	"eta/eta_index_lib/utils"
	"fmt"
	"strconv"
	"time"
)

// BloombergController Bloomberg
type BloombergController struct {
	BaseAuthController
}

// Add
// @Title 新增彭博指标接口
// @Description 新增彭博指标接口
// @Success 200 {object} models.AddEdbInfoReq
// @router /add [post]
func (this *BloombergController) Add() {
	br := new(models.BaseResponse).Init()
	var cacheKey string
	defer func() {
		if br.ErrMsg == "" {
			br.IsSendEmail = false
		}
		_ = utils.Rc.Delete(cacheKey)
		this.Data["json"] = br
		this.ServeJSON()
	}()
	source := utils.DATA_SOURCE_BLOOMBERG
	var req models.AddEdbInfoReq
	err := json.Unmarshal(this.Ctx.Input.RequestBody, &req)
	if err != nil {
		br.Msg = "参数解析异常！"
		br.ErrMsg = "参数解析失败，Err:" + err.Error()
		return
	}
	if req.EdbCode == "" {
		br.Msg = "请输入指标编码！"
		br.ErrMsg = "请输入指标编码，指标编码为空"
		return
	}
	if req.Source != source {
		br.Msg = "数据源有误"
		br.ErrMsg = fmt.Sprintf("数据源ID不匹配, Source: %d", req.Source)
		return
	}
	cacheKey = utils.CACHE_EDB_DATA_ADD + strconv.Itoa(source) + "_" + req.EdbCode
	if !utils.Rc.IsExist(cacheKey) {
		utils.Rc.SetNX(cacheKey, 1, 1*time.Minute)
		err = models.AddEdbDataFromBloomberg(req.EdbCode)
		if err != nil {
			br.Msg = "获取指标信息失败！"
			br.ErrMsg = "获取指标信息失败 AddEdbDataFromBloomberg，Err:" + err.Error()
			return
		}
		br.Ret = 200
		br.Success = true
		br.Msg = "获取成功"
	} else {
		br.Ret = 501
		br.Success = true
		br.Msg = "系统处理中，请稍后重试"
	}
}

// Refresh
// @Title 刷新彭博指标接口
// @Description 刷新彭博指标接口
// @Success 200 {object} models.RefreshEdbInfoReq
// @router /refresh [post]
func (this *BloombergController) Refresh() {
	br := new(models.BaseResponse).Init()
	var cacheKey string
	defer func() {
		if br.ErrMsg == "" {
			br.IsSendEmail = false
		}
		_ = utils.Rc.Delete(cacheKey)
		this.Data["json"] = br
		this.ServeJSON()
	}()
	source := utils.DATA_SOURCE_BLOOMBERG
	var req models.RefreshEdbInfoReq
	err := json.Unmarshal(this.Ctx.Input.RequestBody, &req)
	if err != nil {
		br.Msg = "参数解析异常！"
		br.ErrMsg = "参数解析失败，Err:" + err.Error()
		return
	}
	if req.EdbCode == "" {
		br.Msg = "请输入指标编码！"
		br.ErrMsg = "请输入指标编码，指标编码为空"
		return
	}
	if req.EdbInfoId <= 0 {
		br.Msg = "请输入指标ID！"
		br.ErrMsg = "请输入指标ID"
		return
	}

	// 获取指标详情
	edbInfo, err := models.GetEdbInfoByEdbCode(source, req.EdbCode)
	if err != nil {
		br.Msg = "指标不存在！"
		br.ErrMsg = "指标不存在"
		return
	}
	cacheKey = utils.CACHE_EDB_DATA_REFRESH + strconv.Itoa(source) + "_" + req.EdbCode
	if utils.Rc.IsExist(cacheKey) {
		br.Ret = 501
		br.Success = true
		br.Msg = "系统处理中，请稍后重试"
		return
	}
	dataUpdateTime := time.Now().Format(utils.FormatDateTime)
	utils.Rc.SetNX(cacheKey, 1, 1*time.Minute)
	err = models.RefreshEdbDataFromBloomberg(req.EdbInfoId, req.EdbCode, req.StartDate)
	if err != nil && err.Error() != utils.ErrNoRow() {
		br.Msg = "刷新指标信息失败！"
		br.ErrMsg = "刷新指标信息失败 RefreshEdbDataFromBloomberg，Err:" + err.Error()
		return
	}

	// 更新指标最大最小值
	erDataUpdateDate, err, errMsg := models.UnifiedModifyEdbInfoMaxAndMinInfoDataUpdate(edbInfo, dataUpdateTime)
	if err != nil {
		br.Msg = errMsg
		br.ErrMsg = err.Error()
		return
	}
	// 添加指标刷新成功日志
	if erDataUpdateDate != "" {
		_ = services.AddEdbInfoUpdateLog(edbInfo.EdbInfoId, 1, "", 1, "", 0, 0)
	} else {
		_ = services.AddEdbInfoUpdateLog(edbInfo.EdbInfoId, 1, "", 2, "未刷新到数据", 0, 0)
	}

	// 更新ES
	go logic.UpdateEs(edbInfo.EdbInfoId)

	br.Ret = 200
	br.Success = true
	br.Msg = "获取成功"
}

// PCSGImportHistoryData
// @Title 中石油新加坡-导入历史数据
// @Description  中石油新加坡-导入历史数据
// @Success 200 {object} models.PCSGImportHistoryDataReq
// @router /pcsg/import_history_data [post]
func (this *BloombergController) PCSGImportHistoryData() {
	br := new(models.BaseResponse).Init()
	defer func() {
		if br.ErrMsg == "" {
			br.IsSendEmail = false
		}
		this.Data["json"] = br
		this.ServeJSON()
	}()
	var req models.PCSGImportHistoryDataReq
	if e := json.Unmarshal(this.Ctx.Input.RequestBody, &req); e != nil {
		br.Msg = "参数解析异常！"
		br.ErrMsg = "参数解析失败，Err:" + e.Error()
		return
	}
	if req.TaskKey == "" {
		br.Msg = "TaskKey不可为空"
		return
	}
	tasks, e := services.LoadPCSGBloombergTask()
	if e != nil {
		br.Msg = "加载配置失败"
		br.ErrMsg = fmt.Sprintf("加载配置失败, Err: %v", e)
		return
	}
	conf := new(services.PCSGBloombergTask)
	for _, v := range tasks {
		if v.TaskKey == req.TaskKey {
			conf = v
			break
		}
	}
	if conf.TaskKey == "" {
		br.Msg = "TaskKey配置不存在"
		return
	}

	var indexes []models.BaseFromBloombergApiIndexAndData
	var index models.BaseFromBloombergApiIndexAndData
	var indexData []models.BaseFromBloombergApiIndexData
	index.IndexCode = req.IndexCode
	for k, v := range req.DataMap {
		indexData = append(indexData, models.BaseFromBloombergApiIndexData{
			DataTime: k,
			Value:    v,
		})
	}
	index.Data = indexData
	indexes = append(indexes, index)

	// 写入数据
	if e := services.PCSGWrite2BaseBloomberg(indexes, conf.VCode, conf.ExtraLetter, conf.IndexNamePrefix, conf.IndexCodeSuffix); e != nil {
		br.Msg = "刷新失败"
		br.ErrMsg = "PCSG-写入Bloomberg数据源失败, Err: " + e.Error()
		return
	}

	br.Ret = 200
	br.Success = true
	br.Msg = "操作成功"
}

// PCSGRefreshTask
// @Title 中石油新加坡-刷新任务
// @Description  中石油新加坡-刷新任务
// @Success 200 {object} models.AddEdbInfoReq
// @router /pcsg/refresh_task [post]
func (this *BloombergController) PCSGRefreshTask() {
	br := new(models.BaseResponse).Init()
	defer func() {
		if br.ErrMsg == "" {
			br.IsSendEmail = false
		}
		this.Data["json"] = br
		this.ServeJSON()
	}()
	// 非必传, 只是手动请求的时候用
	var req struct {
		TaskKey string
	}
	_ = json.Unmarshal(this.Ctx.Input.RequestBody, &req)

	tasks, e := services.LoadPCSGBloombergTask()
	if e != nil {
		br.Msg = "加载配置失败"
		br.ErrMsg = fmt.Sprintf("加载配置失败, Err: %v", e)
		return
	}

	for _, v := range tasks {
		if req.TaskKey != "" && v.TaskKey != req.TaskKey {
			continue
		}
		fmt.Println(v)
		time.Sleep(5 * time.Second)

		// 获取数据
		var r services.PCSGBloombergApiReq
		r.TaskKey = v.TaskKey
		r.Frequency = v.Frequency
		indexes, e := services.GetPCSGBloombergGeneralIndexFromBridge(r)
		if e != nil {
			br.Msg = "刷新失败"
			br.ErrMsg = "Bridge-GetPCSGBloombergGeneralIndexFromBridge, Err: " + e.Error()
			return
		}
		if len(indexes) == 0 {
			br.Ret = 200
			br.Success = true
			br.Msg = "操作成功"
			return
		}

		// 此处重新整理一下, 将同一指标的数据合并, 否则会使刷新时间变得很长
		var newIndexes []models.BaseFromBloombergApiIndexAndData
		codeMap := make(map[string]models.BaseFromBloombergApiIndexAndData)
		indexCodeData := make(map[string][]models.BaseFromBloombergApiIndexData)
		for _, iv := range indexes {
			if codeMap[iv.IndexCode].IndexCode == "" {
				var indexItem models.BaseFromBloombergApiIndexAndData
				indexItem.IndexCode = iv.IndexCode
				indexItem.IndexName = iv.IndexName
				indexItem.Unit = iv.Unit
				indexItem.Source = iv.Source
				indexItem.Frequency = iv.Frequency
				indexItem.CreateTime = iv.CreateTime
				indexItem.ModifyTime = iv.ModifyTime
				codeMap[iv.IndexCode] = indexItem
			}
			indexCodeData[iv.IndexCode] = append(indexCodeData[iv.IndexCode], iv.Data...)
		}
		for _, mv := range codeMap {
			mv.Data = indexCodeData[mv.IndexCode]
			newIndexes = append(newIndexes, mv)
		}

		// 写入数据
		if e = services.PCSGWrite2BaseBloomberg(newIndexes, v.VCode, v.ExtraLetter, v.IndexNamePrefix, v.IndexCodeSuffix); e != nil {
			br.Msg = "刷新失败"
			br.ErrMsg = "PCSG-写入Bloomberg数据源失败, Err: " + e.Error()
			return
		}
	}

	br.Ret = 200
	br.Success = true
	br.Msg = "操作成功"
}
